unit PropertyEditor;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms,
  ExtCtrls, StdCtrls, Spin;

type
  TfrmPropertyPage = class(TForm)
    Panel1: TPanel;
    edtCaption: TEdit;
    Label1: TLabel;
    pnFieldMod: TPanel;
    rgFieldType: TRadioGroup;
    pnDictionary: TPanel;
    HintTime: TTimer;
    pnFieldName: TPanel;
    chxRequired: TCheckBox;
    chxCanModify: TCheckBox;
    Label3: TLabel;
    cbxFieldName: TComboBox;
    Label4: TLabel;
    pnTabOrder: TPanel;
    seTabOrder: TSpinEdit;
    Label2: TLabel;
    pnRelation: TPanel;
    Label5: TLabel;
    cbxRelation: TComboBox;
    cbxDictionary: TComboBox;
    procedure edtCaptionChange(Sender: TObject);
    procedure chxCanModifyClick(Sender: TObject);
    procedure rgFieldTypeClick(Sender: TObject);
    procedure seTabOrderChange(Sender: TObject);
    procedure edKeyFieldChange(Sender: TObject);
    procedure edtIdentExit(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HintTimeTimer(Sender: TObject);
    procedure edtIdentKeyPress(Sender: TObject; var Key: Char);
    procedure cbxDictionaryChange(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure cbxFieldNameClick(Sender: TObject);
    procedure cbxFieldNameKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxRelationChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FFieldControl:TControl;
    FStaticText:TControl;
    FHWin:THintWindow;
    procedure Refill(Sender: TObject);
    function FillDictionaryText(AFieldName:String):String;
    procedure LoadUpUsableRelations(AFieldType:integer;ADictionaryField:Boolean);
  end;

var
  frmPropertyPage: TfrmPropertyPage;

implementation

{$R *.DFM}
uses TypInfo, ZEDit,ZEDPicker,ZEDChxBox, ZEDText, Buttons, Sysutils,
  FormEditClientMain, ComponentPalette, ClientMainDataModule, DBGrids, DB, variants;
{ TfrmPropertyPage }

procedure TfrmPropertyPage.Refill(Sender: TObject);
  function isNull(AExpVal,ResIfTrue:Variant):Variant;
  Begin
    if VarIsNull(AExpVal) then
      Result:=ResIfTrue
    else
      Result:=AExpVal;
  End;
var cv:integer;
    lvKeyField,lvDictionary:String;
    lvFieldType:integer;
begin
  if Sender is TZEDText then
  Begin
    FFieldControl:=TZEDText(Sender).FocusControl;
    FStaticText:=TControl(Sender);
  End
  else
  if assigned(Sender) and not(Sender is TZEDChxBox) and not(Sender is TBitBtn) then
  Begin
    FFieldControl:=TControl(Sender);
    for cv:=0 to TControl(Sender).Parent.ControlCount-1 do
      if (TControl(Sender).Parent.Controls[cv] is TZEDText) and
         (TZEDText(TControl(Sender).Parent.Controls[cv]).FocusControl=Sender) then
      Begin
        FStaticText:=TControl(Sender).Parent.Controls[cv];
        break;
      End;
  End
  else
  Begin
    FFieldControl:=TControl(Sender);
    FStaticText:=TControl(Sender);
  End;

  if assigned(FStaticText) then
    edtCaption.Text:=VarToStr(GetPropValue(FStaticText,'Caption'))
  else
    edtCaption.Text:='';
  edtCaption.Enabled:=assigned(FFieldControl) or assigned(FStaticText);

  if assigned(FFieldControl) then
  Begin
    pnFieldMod.Visible:=TWinControl(FFieldControl).Parent.Name='tsEdit';
    pnRelation.Visible:=not pnFieldMod.Visible;

    try
      chxRequired.Checked:=isNull(GetPropValue(FFieldControl,'Required'),false);
    except
      chxRequired.Checked:=false;
    end;
    try
      chxCanModify.Checked:=isNull(GetPropValue(FFieldControl,'CanModify'),false);
    except
      chxCanModify.Checked:=false;
    end;
    try
      seTabOrder.Value:=GetPropValue(FFieldControl,'TabOrder');
    except
      seTabOrder.Value:=0;
    end;

    cbxRelation.Enabled:=not(FFieldControl is TDBGrid) and not(FFieldControl is TZEDText)
      and not(FFieldControl is TBitBtn);

    try
      lvKeyField:=VarToStr(GetPropValue(FFieldControl,'KeyField'));
    except
      lvKeyField:='';
    end;
    try
      lvDictionary:=FillDictionaryText(lvKeyField);
    except
      lvDictionary:='';
    end;
    try
      lvFieldType:=isNull(GetPropValue(FFieldControl,'FieldType'),-1);
    except
      lvFieldType:=-1;
    end;
    if (lvDictionary<>'') and (lvFieldType=0) then
      rgFieldType.ItemIndex:=1
    else
      rgFieldType.ItemIndex:=lvFieldType;

    if pnRelation.Visible then
    Begin
      case rgFieldType.ItemIndex of
        -1: Begin End;
         2: dmClientMain.FillFieldList(cbxFieldName.Items,frmClientMain.FFList[rgFieldType.ItemIndex]);
         3: dmClientMain.FillFieldList(cbxFieldName.Items,nil)
         else
           dmClientMain.FillFieldList(cbxFieldName.Items,frmClientMain.FFList[0],frmClientMain.FFList[1]);
      End;
      cbxFieldName.Style:=csDropDownList;
      try
        cbxFieldName.ItemIndex:=cbxFieldName.Items.IndexOf(VarToStr(lvKeyField));
      except
        cbxFieldName.ItemIndex:=-1;
      end;
      cbxDictionary.Text:=lvDictionary;
    End
    else
    Begin
      if (rgFieldType.ItemIndex>-1) then
        dmClientMain.FillFieldList(cbxFieldName.Items,frmClientMain.FFList[rgFieldType.ItemIndex]);
      cbxFieldName.Style:=csDropDown;
      try
        cbxFieldName.Text:=VarToStr(GetPropValue(FFieldControl,'KeyField'));
      except
        cbxFieldName.Text:='';
      end;
      try
        cbxDictionary.Text:=VarToStr(GetPropValue(FFieldControl,'Dictionary'));
      except
        cbxDictionary.Text:='';
      end;
    End;

    try
      cbxFieldName.Enabled:=(cbxRelation.Enabled and
      (pnRelation.Visible or not Boolean(dmClientMain.GetFieldType(LowerCase(cbxFieldName.Text))+1) or (GetPropValue(FFieldControl,'Tag')=1) ));
    except
      cbxFieldName.Enabled:=(cbxRelation.Enabled and pnRelation.Visible);
    end;
    rgFieldType.Enabled:=(FFieldControl is TZEDit) and not pnRelation.Visible;

    if (FFieldControl is TZEDit) then
    Begin
      rgFieldType.Controls[0].Enabled:=cbxFieldName.Enabled and not pnRelation.Visible;
      rgFieldType.Controls[1].Enabled:=cbxFieldName.Enabled and not pnRelation.Visible;
      rgFieldType.Controls[2].Enabled:=False;
      rgFieldType.Controls[3].Enabled:=False;
    End;

    cbxDictionary.Enabled:=rgFieldType.Enabled and cbxFieldName.Enabled and not pnRelation.Visible;
    chxRequired.Enabled:=(FFieldControl is TZEDPicker) or (FFieldControl is TZEDit);
    chxCanModify.Enabled:=chxRequired.Enabled or (FFieldControl is TZEDChxBox);
    seTabOrder.Enabled:=not(FFieldControl is TZEDText);

    if cbxDictionary.Enabled then
      dmClientMain.FillDictList(cbxDictionary.Items);

    if pnRelation.Visible then
    Begin
      LoadUpUsableRelations(rgFieldType.ItemIndex,cbxDictionary.Text<>'');
      try
        cbxRelation.ItemIndex:=cbxRelation.Items.IndexOf(VarToStr(GetPropValue(FFieldControl,'Relation')));
      except
        cbxRelation.ItemIndex:=-1;
      end;
    End;
  End
  else
  Begin
    cbxFieldName.ItemIndex:=-1;
    cbxFieldName.Text:='';
    cbxFieldName.Enabled:=False;
    rgFieldType.ItemIndex:=-1;
    rgFieldType.Enabled:=False;
    cbxDictionary.ItemIndex:=-1;
    cbxDictionary.Text:='';
    cbxDictionary.Enabled:=false;
    chxRequired.Checked:=false;
    chxRequired.Enabled:=false;
    chxCanModify.Checked:=false;
    chxCanModify.Enabled:=false;
    if assigned(FStaticText) then
      seTabOrder.Value:=GetPropValue(FStaticText,'TabOrder');
    seTabOrder.Enabled:=False;
  End;
end;

procedure TfrmPropertyPage.edtCaptionChange(Sender: TObject);
begin
  if TWinControl(Sender).Focused and assigned(FStaticText) and (edtCaption.Focused) then
  if TEdit(Sender).Text<>'' then
    SetPropValue(FStaticText,'Caption',TEdit(Sender).Text)
  else
    SetPropValue(FStaticText,'Caption','[]');
end;

procedure TfrmPropertyPage.chxCanModifyClick(Sender: TObject);
begin
  if TWinControl(Sender).Focused and assigned(FFieldControl) then
    SetPropValue(FFieldControl,Copy(TControl(Sender).Name,4,length(TControl(Sender).Name)-3),BooleanIdents[TCheckBox(Sender).Checked]);
end;

procedure TfrmPropertyPage.rgFieldTypeClick(Sender: TObject);
begin
  if assigned(FFieldControl) then
  Begin
    if (FFieldControl is TZEDit) then
      SetPropValue(FFieldControl,'FieldType',TRadioGroup(Sender).ItemIndex);
    dmClientMain.FillFieldList(cbxFieldName.Items,frmClientMain.FFList[TRadioGroup(Sender).ItemIndex]);
    LoadUpUsableRelations(TRadioGroup(Sender).ItemIndex,GetPropValue(FFieldControl,'Dictionary')<>'');
  End;
end;

procedure TfrmPropertyPage.seTabOrderChange(Sender: TObject);
begin
  if TWinControl(Sender).Focused and assigned(FFieldControl) then
  try
    SetPropValue(FFieldControl,'TabOrder',TSpinEdit(Sender).Value);
  except
  end;
end;

procedure TfrmPropertyPage.edKeyFieldChange(Sender: TObject);
begin
  if (ActiveControl=Sender) and assigned(FFieldControl) then
  try
    SetPropValue(FFieldControl,'KeyField',TComboBox(Sender).Text);
  except
  end;
end;

procedure TfrmPropertyPage.edtIdentExit(Sender: TObject);
var lvPos:TPoint;
    lvRect:TRect;
    lvidx:integer;
function isKeyFieldControlExist(AFieldName:TCaption):boolean;
var cv:integer;
Begin
  Result:=False;
  for cv:=0 to TWinControl(FFieldControl).Parent.ControlCount-1 do
  try
    if (TWinControl(FFieldControl).Parent.Controls[cv]<>FFieldControl) and (LowerCase(VarToStr(GetPropValue(TWinControl(FFieldControl).Parent.Controls[cv],'KeyField')))=AFieldName) then
    Begin
      Result:=True;
      Break;
    End
  except
  end;
End;
begin
  if TWinControl(Sender).Enabled and (GetPropValue(Sender,'Text')<>'') and
  (
    ( (TWinControl(FFieldControl).Parent.Name='tsEdit') and
      ( not IsValidIdent(GetPropValue(Sender,'Text')) or isKeyFieldControlExist(LowerCase(GetPropValue(Sender,'Text'))) ) )or
    ( (TWinControl(FFieldControl).Parent.Name='tsSearch') and not Boolean(dmClientMain.GetFieldType(LowerCase(GetPropValue(Sender,'Text')))+1) )
  ) then
  Begin
    SetFocus;
    TWinControl(Sender).SetFocus;
    lvPos.x:=TWinControl(Sender).Left;
    lvPos.y:=TWinControl(Sender).Top+TWinControl(Sender).Height;
    lvPos:=TWinControl(TWinControl(Sender).Parent).ClientToScreen(lvPos);
    if not FHWin.Visible then
    Begin
      FHWin.Hint:='Az azonost rvnytelen';
      lvRect:=FHWin.CalcHintRect(TWinControl(Sender).Width,FHWin.Hint,nil);
      lvRect.Left:=lvPos.x;
      lvRect.Top:=lvPos.y;
      lvRect.Right:=lvRect.Right+lvPos.x;
      lvRect.Bottom:=lvRect.Bottom+lvPos.y;
      FHWin.ActivateHint(lvRect,FHWin.Hint);
      FHWin.Show;
      HintTime.Enabled:=True;
      if (rgFieldType.ItemIndex in [0,1]) then
      Begin
        rgFieldType.OnClick:=nil;
        lvidx:=dmClientMain.GetFieldType(LowerCase(TComboBox(Sender).Text));
        if Boolean(lvidx+1) then
          if (lvidx=0) and (cbxDictionary.Text<>'') then
            rgFieldType.ItemIndex:=1
          else
            rgFieldType.ItemIndex:=lvidx;
        rgFieldType.OnClick:=rgFieldTypeClick;
      End;
    End;
  End;
end;

procedure TfrmPropertyPage.FormDestroy(Sender: TObject);
begin
  FHWin.Free;
end;

procedure TfrmPropertyPage.FormCreate(Sender: TObject);
begin
  FHWin:=THintWindow.Create(Self);
  FHWin.Hide;
end;

procedure TfrmPropertyPage.HintTimeTimer(Sender: TObject);
begin
  FHWin.ReleaseHandle;
  FHWin.Hide;
  HintTime.Enabled:=False;
end;

procedure TfrmPropertyPage.edtIdentKeyPress(Sender: TObject;
  var Key: Char);
begin
  if FHWin.Visible then
  Begin
    FHWin.Hide;
    FHWin.ReleaseHandle;
  End;
end;

procedure TfrmPropertyPage.cbxDictionaryChange(Sender: TObject);
begin
  if assigned(FFieldControl) and (FFieldControl is TZEDit) then
    SetPropValue(FFieldControl,'Dictionary',TComboBox(Sender).Text);
end;

procedure TfrmPropertyPage.FormDeactivate(Sender: TObject);
begin
  edtIdentExit(cbxFieldName);
end;

procedure TfrmPropertyPage.LoadUpUsableRelations(AFieldType: integer;ADictionaryField:Boolean);
begin
  cbxRelation.Clear;
  if AFieldType<>3 then
  Begin
    cbxRelation.Items.Add('Egyenl');
    cbxRelation.Items.Add('NemEgyenl');
  End
  else
  Begin
    cbxRelation.Items.Add('res');
    cbxRelation.Items.Add('Nemres');
  End;
  if (AFieldType in [0,2]) and not ADictionaryField then
  Begin
    cbxRelation.Items.Add('Kisebb');
    cbxRelation.Items.Add('Nagyobb');
    cbxRelation.Items.Add('KisebbEgyenl');
    cbxRelation.Items.Add('NagyobbEgyenl');
  End;
  if (AFieldType = 1) or (ADictionaryField and (AFieldType = 0)) then
  Begin
    cbxRelation.Items.Add('Hasonl');
    cbxRelation.Items.Add('NemHasonl');
  End;
end;

procedure TfrmPropertyPage.cbxFieldNameClick(Sender: TObject);
var lvidx:integer;
begin
  cbxDictionary.Text:=FillDictionaryText(cbxFieldName.Text);
  if (rgFieldType.ItemIndex in [0,1]) then
  Begin
    rgFieldType.OnClick:=nil;
    lvidx:=dmClientMain.GetFieldType(LowerCase(TComboBox(Sender).Text));
    if Boolean(lvidx+1) then
      if (lvidx=0) and (cbxDictionary.Text<>'') then
        rgFieldType.ItemIndex:=1
      else
        rgFieldType.ItemIndex:=lvidx;
    rgFieldType.OnClick:=rgFieldTypeClick;
  End;
  cbxDictionaryChange(cbxDictionary);
  LoadUpUsableRelations(rgFieldType.ItemIndex,cbxDictionary.Text<>'');
end;

procedure TfrmPropertyPage.cbxFieldNameKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key=VK_DELETE) and (TComboBox(Sender).Style=csDropDownList) then
    TComboBox(Sender).ItemIndex:=-1;
end;

function TfrmPropertyPage.FillDictionaryText(AFieldName: String): String;
var lvDictGroupID:Variant;
begin
  with dmClientMain.qrySearch do
  Begin
    Result:='';
    Close;
    SQL.Text:=CFieldDictionary;
    Open;
    if Locate('FieldName',AFieldName,[loCaseInsensitive]) then
    Begin
      lvDictGroupID:=FieldByName('DictionaryGroupID').Value;
      if not VarIsNull(lvDictGroupID) then
      Begin
        Close;
        SQL.Text:=CDictionaryGroup;
        Open;
        if Locate('DictionaryGroupID',lvDictGroupID,[]) then
          Result:=FieldByName('DictionaryGroupName').Value;
      End;
    End;
  End;
end;

procedure TfrmPropertyPage.cbxRelationChange(Sender: TObject);
begin
  if assigned(FFieldControl) then
    SetPropValue(FFieldControl,'Relation',TComboBox(Sender).Text);
end;

end.
